namespace StudySystemClient {
	public class ActivitiesView : CardArea {
		private Client client;

		public ActivitiesView(Client client)  {
			base();

			this.client = client;
			this.map.connect(refresh);
		}

		private async void refresh() {
			try {
				var activities = yield client.list_activities();
				clear();
				foreach (var activity in activities) {
					var card = new ActivityCard(activity);
					card.session_logged.connect(log_session);
					add(card);
				}
			} catch (ClientError e) {
				stderr.printf("Error refreshing activities: %s\n",
							  e.message);
			}
		}

		private async void log_session(string subject, ActivityType type,
									   int minutes) {
			try {
				yield client.log_session(subject, type, minutes);
				yield refresh();
			} catch (ClientError e) {
				stderr.printf("Error logging session: %s\n", e.message);
			}
		}
	}
}
