namespace StudySystemClient {
	public class ActivityCard : Card {
		public signal void session_logged(string subject, ActivityType type,
										  int minutes);

		public ActivityCard(Activity activity) {
			base();

			var subject = new Gtk.Label(activity.subject);
			subject.halign = Gtk.Align.START;
			subject.add_css_class("activity-subject");

			var type = new Gtk.Label(activity.type.to_string());
			type.add_css_class("activity-type");
			var separator = new Gtk.Label("·");
			separator.add_css_class("activity-priority");
			var priority = new Gtk.Label("%0.2f".printf(activity.priority));
			priority.add_css_class("activity-priority");

			var details = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);
			details.append(type);
			details.append(separator);
			details.append(priority);

			var text = new Gtk.Box(Gtk.Orientation.VERTICAL, 6);
			text.hexpand = true;
			text.append(subject);
			text.append(details);

			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 12);
			content.append(text);

			var button
				= new Gtk.Button.from_icon_name("appointment-new-symbolic");
			button.vexpand = false;
			button.valign = Gtk.Align.CENTER;
			button.set_tooltip_text("Log session");
			button.add_css_class("log-session-button");
			content.append(button);

			set_child(content);

			var log_session_popover = new LogSessionPopover();
			log_session_popover.set_parent(button);
			button.clicked.connect(() => log_session_popover.popup());
			log_session_popover.session_logged.connect((minutes) => {
					session_logged(activity.subject, activity.type, minutes);
				});
		}
	}

	private class LogSessionPopover : Gtk.Popover {
		public signal void session_logged(int minutes);

		private const int DEFAULT_LENGTH = 30;

		private Gtk.SpinButton input;

		public LogSessionPopover() {
			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);

			var label = new Gtk.Label("Minutes");
			label.halign = Gtk.Align.START;
			content.append(label);

			var adjustment
				= new Gtk.Adjustment(DEFAULT_LENGTH, 10, 480, 10, 10, 0);
			input = new Gtk.SpinButton(adjustment, 1, 0);
			input.numeric = true;
			content.append(input);

			var button = new Gtk.Button.from_icon_name("emblem-ok-symbolic");
			button.halign = Gtk.Align.END;
			button.set_tooltip_text("Submit");
			button.add_css_class("suggested-action");
			button.clicked.connect(submit);
			content.append(button);

			set_child(content);

			closed.connect(reset);
		}

		private void submit() {
			session_logged((int)input.value);
			reset();
			popdown();
		}

		private void reset() {
			input.value = DEFAULT_LENGTH;
		}
	}
}
